"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RepackPlugin = void 0;
var _BabelPlugin = require("./BabelPlugin.js");
var _CodegenPlugin = require("./CodegenPlugin.js");
var _DevelopmentPlugin = require("./DevelopmentPlugin.js");
var _LoggerPlugin = require("./LoggerPlugin.js");
var _NativeEntryPlugin = require("./NativeEntryPlugin.js");
var _index = require("./OutputPlugin/index.js");
var _index2 = require("./RepackTargetPlugin/index.js");
/**
 * {@link RepackPlugin} configuration options.
 */

/**
 * Webpack plugin, which abstracts configuration of other Re.Pack's plugin
 * to make Webpack config more readable.
 *
 * @example Usage in Webpack config (ESM):
 * ```ts
 * import * as Repack from '@callstack/repack';
 *
 * export default (env) => {
 *   const {
 *     mode = 'development',
 *     platform,
 *     devServer = undefined,
 *   } = env;
 *
 *   return {
 *     plugins: [
 *       new Repack.RepackPlugin({
 *         mode,
 *         platform,
 *         devServer,
 *       }),
 *     ],
 *   };
 * };
 * ```
 *
 * Internally, `RepackPlugin` configures the following plugins:
 * - `webpack.DefinePlugin` with `__DEV__` global
 * - {@link AssetsResolverPlugin}
 * - {@link OutputPlugin}
 * - {@link DevelopmentPlugin}
 * - {@link RepackTargetPlugin}
 * - `webpack.SourceMapDevToolPlugin`
 * - {@link LoggerPlugin}
 *
 * `RepackPlugin` provides a sensible defaults, but can be customized to some extent.
 * If you need more control, it's recommended to remove `RepackPlugin` and use other plugins
 * directly, eg:
 * ```ts
 * import * as Repack from '@callstack/repack';
 *
 * new Repack.plugins.AssetsResolverPlugin();
 * ```
 *
 * @category Webpack Plugin
 */
class RepackPlugin {
  /**
   * Constructs new `RepackPlugin`.
   *
   * @param config Plugin configuration options.
   */
  constructor(config) {
    this.config = config;
    this.config.sourceMaps = this.config.sourceMaps ?? true;
    this.config.logger = this.config.logger ?? true;
  }
  getEntryName(compiler) {
    if (this.config.entryName) {
      return this.config.entryName;
    }
    if (typeof compiler.options.entry === 'object' && 'main' in compiler.options.entry) {
      return 'main';
    }
    return undefined;
  }

  /**
   * Apply the plugin.
   *
   * @param compiler Webpack compiler instance.
   */
  apply(compiler) {
    const entryName = this.getEntryName(compiler);
    new compiler.webpack.DefinePlugin({
      __DEV__: JSON.stringify(this.config.mode === 'development')
    }).apply(compiler);
    new _BabelPlugin.BabelPlugin().apply(compiler);
    new _CodegenPlugin.CodegenPlugin().apply(compiler);
    new _index.OutputPlugin({
      platform: this.config.platform,
      enabled: !this.config.devServer && !!entryName,
      context: this.config.context,
      output: this.config.output,
      entryName: this.config.entryName,
      extraChunks: this.config.extraChunks
    }).apply(compiler);
    if (entryName) {
      new _NativeEntryPlugin.NativeEntryPlugin({
        entryName,
        initializeCoreLocation: this.config.initializeCore
      }).apply(compiler);
    }
    new _index2.RepackTargetPlugin({
      hmr: this.config.devServer?.hmr
    }).apply(compiler);
    new _DevelopmentPlugin.DevelopmentPlugin({
      devServer: this.config.devServer,
      entryName,
      platform: this.config.platform
    }).apply(compiler);
    if (this.config.sourceMaps) {
      // TODO Fix sourcemap directory structure
      // Right now its very messy and not every node module is inside of the node module
      // like React Devtools backend etc or some symilinked module appear with relative path
      // We should normalize this through a custom handler and provide an output similar to Metro
      new compiler.webpack.SourceMapDevToolPlugin({
        test: /\.(js)?bundle$/,
        filename: '[file].map',
        append: `//# sourceMappingURL=[url]?platform=${this.config.platform}`,
        module: true,
        columns: true,
        noSources: false,
        namespace: compiler.options.output.devtoolNamespace ?? compiler.options.output.uniqueName
      }).apply(compiler);
    }
    if (this.config.logger) {
      new _LoggerPlugin.LoggerPlugin({
        platform: this.config.platform,
        devServerEnabled: Boolean(this.config.devServer),
        output: {
          console: true,
          ...(typeof this.config.logger === 'object' ? this.config.logger : {})
        }
      }).apply(compiler);
    }
  }
}
exports.RepackPlugin = RepackPlugin;
//# sourceMappingURL=RepackPlugin.js.map